"""Configuration for custom attributes generator."""

STATIC_IDS_JSON_FILE = "special_attributes.json"
JSON_DATA_FILE_NAME = "json_data"

ATTRIBUTE_QUERY = {"active-eq": True}
DEFAULT_GROUP_NAME = "~ no group"

# --------- Generate Python Code ---------
BASE_ATTRIBUTES = ["id", "valueName"]
